package Alignment;

use strict;
use warnings;

use Error;

###########################################################################
# constructor
###########################################################################

sub new {
  my $class = shift(@_);
  $class = ref($class) || $class;

  if (@_ != 0) {
    Error::error('Wrong argument number');
  }

  my @parts = ();

  my $self = {
    'parts' => \@parts,
    'score' => 0,
  };

  bless($self, $class);

  return($self);
}


###########################################################################
# methods
###########################################################################

sub str {
  my $self = shift(@_);

  if (@_ > 1) {
    Error::error('Wrong argument number');
  }

  my $indent = 0;
  if (@_ == 1) {
    $indent = shift(@_);
  }
  my $ind = " " x $indent;
  my $string = '';

  $string .= $ind.'parts:'."\n";
  for (my $i = 0; $i < @{$self->{'parts'}}; ++$i) {
    $string .= $ind.'  part '.($i + 1).':'."\n";
    $string .= $self->{'parts'}->[$i]->str($indent + 4);
  }
  $string .= $ind.'score:'."\n";
  $string .= $ind.'  '.$self->{'score'}."\n";

  return ($string);
}

sub append_part {
  my $self = shift(@_);
  
  if (@_ != 1) {
    Error::error('Wrong argument number');
  }

  my $part = $_[0];

  push(@{$self->{'parts'}}, $part);
}

sub set_score {
  my $self = shift(@_);
  
  if (@_ != 1) {
    Error::error('Wrong argument number');
  }

  my $score = $_[0];

  $self->{'score'} = $score;
}

sub length {
  my $self = shift(@_);
  
  if (@_) {
    Error::error('Wrong argument number');
  }

  if (@{$self->{'parts'}} <= 0) {
    return(0);
  }
  else {
    return(length($self->{'parts'}->[0]->{'seq'}));
  }
}

sub parts_size {
  my $self = shift(@_);
  
  if (@_) {
    Error::error('Wrong argument number');
  }

  my $size = @{$self->{'parts'}};
  
  return($size);
}

sub part {
  my $self = shift(@_);
  
  if (@_ != 1) {
    Error::error('Wrong argument number');
  }

  my $index = $_[0];
  
  unless ($index < @{$self->{'parts'}}) {
    my $message = 'No part with index \''.$index.'\' exists';
    Error::debug($message);
  }

  my $part = $self->{'parts'}->[$index];

  return($part);
}

sub score {
  my $self = shift(@_);
  
  if (@_) {
    Error::error('Wrong argument number');
  }

  my $score = $self->{'score'};
  
  return($score);
}

1;
